/**
 * Copyright (C) 2011 Angelo Zerr <angelo.zerr@gmail.com> and Pascal Leclercq <pascal.leclercq@gmail.com>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free  of charge, to any person obtaining
 * a  copy  of this  software  and  associated  documentation files  (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute,  sublicense, and/or sell  copies of  the Software,  and to
 * permit persons to whom the Software  is furnished to do so, subject to
 * the following conditions:
 *
 * The  above  copyright  notice  and  this permission  notice  shall  be
 * included in all copies or substantial portions of the Software.
 *
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY,    FITNESS    FOR    A   PARTICULAR    PURPOSE    AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package fr.opensagres.xdocreport.converter;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import fr.opensagres.xdocreport.converter.discovery.IConverterDiscovery;
import fr.opensagres.xdocreport.core.registry.AbstractRegistry;

/**
 * {@link IConverter} registry.
 * 
 */
public class ConverterRegistry extends AbstractRegistry {

	private static final ConverterRegistry INSTANCE = new ConverterRegistry();
	private static final String CONVERTER_DISCOVERY_PROPERTIES = "converter-discovery.properties";

	private Map<String /* from */, ConverterFrom> converters = new HashMap<String, ConverterFrom>();

	public ConverterRegistry() {
		super(CONVERTER_DISCOVERY_PROPERTIES, ConverterRegistry.class.getName());
	}

	public static ConverterRegistry getRegistry() {
		return INSTANCE;
	}

	public IConverter findConverter(Options options)
			throws XDocConverterException {
		return findConverter(options.getFrom(), options.getTo(),
				options.getVia());
	}

	public IConverter findConverter(String from, String to, String via)
			throws XDocConverterException {
		return internalFindConverter(from, to, via, true);
	}

	public IConverter getConverter(Options options) {
		return getConverter(options.getFrom(), options.getTo(),
				options.getVia());
	}

	public IConverter getConverter(String from, String to, String via) {
		try {
			return internalFindConverter(from, to, via, false);
		} catch (XDocConverterException e) {
			return null;
		}
	}

	public ConverterFrom getConverterFrom(String from) {
		initializeIfNeeded();
		return converters.get(from);
	}

	public Collection<String> getFroms() {
		initializeIfNeeded();
		return converters.keySet();
	}

	private IConverter internalFindConverter(String from, String to,
			String via, boolean throwError) throws XDocConverterException {
		initializeIfNeeded();
		ConverterFrom fromConverters = getConverterFrom(from);
		if (fromConverters == null) {
			if (throwError) {
				throw new XDocConverterException(
						"Cannot find converters for from=" + from);
			}
			return null;
		}
		ConverterTo toConverters = fromConverters.getConverterTo(to);
		if (toConverters == null) {
			if (throwError) {
				throw new XDocConverterException(
						"Cannot find converters for to=" + to + " for from="
								+ from);
			}
			return null;
		}
		if (via == null) {
			IConverter converter = toConverters.getFirstConverter();
			if (throwError && converter == null) {
				throw new XDocConverterException(
						"Cannot find converters for from=" + from + ", to="
								+ to);
			}
			return converter;
		}

		IConverter converter = toConverters.getConverter(via);
		if (throwError && converter == null) {
			throw new XDocConverterException("Cannot find converter via=" + via
					+ " for from=" + from + ", to=" + to);
		}
		return converter;
	}

	public void registerConverterDiscovery(IConverterDiscovery discovery) {
		String from = discovery.getFrom();
		String to = discovery.getTo();
		String via = discovery.getVia();
		IConverter converter = discovery.getConverter();
		ConverterFrom converterFrom = converters.get(from);
		if (converterFrom == null) {
			converterFrom = new ConverterFrom(from);
			converters.put(converterFrom.getFrom(), converterFrom);
		}
		converterFrom.addConverter(to, via, converter);

	}

	@Override
	protected boolean registerInstance(Object instance) {
		if (instance instanceof IConverterDiscovery) {
			registerConverterDiscovery((IConverterDiscovery) instance);
			return true;
		}
		return false;
	}
	
	@Override
	protected void doDispose() {
		this.converters.clear();		
	}
}
